import requests
from bs4 import BeautifulSoup

class CBB:   
 
    def __init__(self):
        self.url = 'https://www.sports-reference.com/cbb'
        self.schools = {}

        response = requests.get(self.url + '/schools')
        soup = BeautifulSoup(response.text, features='lxml')

        for item in soup.find_all(attrs={'data-stat': 'school_name'})[1:510]:
            if item.find('a') is not None:
                self.schools[item.find('a')['href'].split('/')[-2]] = {
                "team_name": item.text,
                "url": 'https://www.sports-reference.com' + item.find('a')['href'],
                } 

    def school_codes(self):
        '''
        Print list of team codes, which are used for getting a specific schools.
        '''
        for abbrev, team_name in self.schools.items():
            print(f"{abbrev} ({team_name['team_name']})")


class CBBSchool(CBB):
    def __init__(self, school):
        super().__init__()
        self.abbreviation = school
        self.school = self.schools[school]['team_name']
        self.school_url = self.schools[school]['url']

    def __repr__(self):
        return f"<{self.abbreviation} - {self.school}>"


import requests
from bs4 import BeautifulSoup

class NFL:   

    def __init__(self):
        self.url = 'https://www.pro-football-reference.com'
        self.teams = {}

        response = requests.get(self.url + '/teams')
        soup = BeautifulSoup(response.text, features='lxml')

        for item in soup.find_all(attrs={'data-stat': 'team_name', 'class':'left'})[1:33]:
            self.teams[item.find('a')['href'].split('/')[-2]] = {
                "team_name": item.text,
                "url": self.url + item.find('a')['href'],
            } 

    def franchise_codes(self):
        '''
        Print list of team codes, which are used for getting a specific franchise.
        '''
        for abbrev, team_name in self.teams.items():
            print(f"{abbrev} ({team_name['team_name']})")


import pandas as pd
from sportrefpy.mlb.league import MLB
import numpy as np

class MLBFranchise(MLB):
    def __init__(self, franchise):
        super().__init__()
        self.abbreviation = franchise
        self.franchise = self.teams[franchise]['team_name']
        self.team_url = self.teams[franchise]['url']
        self.batters_url = self.team_url + 'bat.shtml'
        self.pitchers_url = self.team_url + 'pitch.shtml'
        self.managers_url = self.team_url + 'managers.shtml'
        self.current_roster_url = f'{self.url}/contracts/{self.abbreviation}.html'
        self.seasons_url = f'{self.url}/teams/{self.abbreviation}'

    
    def batters_all_time_stats(self):
        '''
        Returns Pandas dataframe of all historical player data.
        '''

        players = pd.read_html(self.batters_url)[0]
        players.drop(columns={'S', 'C', 'F'}, inplace=True)
        players = players[players.columns[:-1]]
        players.dropna(axis='rows', subset='Name', inplace=True)
        players = players[players['Name'] != 'Name']
        players['Name'] = players['Name'].apply(lambda x: x.split(' HOF')[0])
        players.set_index('Name', inplace=True)
        players = players.apply(pd.to_numeric)
        
        return players

    def pitchers_all_time_stats(self):
        '''
        Returns Pandas dataframe of all historical player data.
        '''

        pitchers = pd.read_html(self.pitchers_url)[0]
        pitchers.drop(columns={'S', 'C', 'F'}, inplace=True)
        pitchers.dropna(axis='rows', subset='Name', inplace=True)
        pitchers = pitchers[pitchers['Name'] != 'Name']
        pitchers['Name'] = pitchers['Name'].apply(lambda x: x.split(' HOF')[0])
        pitchers.set_index('Name', inplace=True)
        pitchers = pitchers.apply(pd.to_numeric)
        
        return pitchers

    def managers_all_time_data(self):
        '''
        Returns Pandas dataframe of all historical coach data.
        '''

        managers = pd.read_html(self.managers_url)[0]
        managers.dropna(axis='rows', subset='Mgr', inplace=True)
        managers.drop(columns={'Rk'}, inplace=True)
        managers = managers[managers['Mgr'] != 'Mgr']
        managers['Mgr'] = managers['Mgr'].apply(lambda x: x.split(' HOF')[0])
        managers.set_index('Mgr', inplace=True)
        managers = managers.apply(pd.to_numeric)

        return managers


    # def current_roster(self):
    #     '''
    #     Returns Pandas dataframe of current roster.
    #     '''

    #     current_roster = pd.read_html(self.current_roster_url)[0]
    #     current_roster = current_roster[current_roster.columns[:2]]
    #     current_roster.columns = ['Player', 'Age']
    #     current_roster = current_roster[current_roster['Player']\
    #          != 'Team Totals']
    #     current_roster.set_index('Player', inplace=True)
    #     current_roster['Age'] = current_roster['Age'].apply(lambda x: int(x))

    #     return current_roster

    
    def season_history(self):
        '''
        Returns Pandas dataframe of seasons.
        '''

        seasons = pd.read_html(self.team_url)[0]
        seasons = seasons[seasons['Tm'] != 'Tm']
        seasons['Year'] = seasons['Year'].astype(int)
        seasons['Playoffs'] = seasons['Playoffs'].astype(str)
        seasons['Playoffs'] = seasons['Playoffs']\
            .apply(lambda x: x.replace('\xa0', ' '))
        seasons['Playoffs'].replace('nan', np.nan, inplace=True)
        seasons.set_index('Year', inplace=True)
        seasons.drop(columns={'Tm'}, inplace=True)

        return seasons

    def __repr__(self):
        return f"<{self.abbreviation} - {self.franchise}>"


from sportrefpy.nhl.league import NHL
import pandas as pd
import numpy as np

class NHLFranchise(NHL):
    def __init__(self, franchise):
        super().__init__()
        self.abbreviation = franchise
        self.franchise = self.teams[franchise]['team_name']
        self.team_url = self.teams[franchise]['url']
        self.skaters_url = self.team_url.replace('history', 'skaters')
        self.goalies_url = self.team_url.replace('history', 'goalies')
        self.coaches_url = self.team_url.replace('history', 'coaches')
        self.current_roster_url = self.team_url.replace('history.html', '')

    
    def skaters_all_time_stats(self):
        '''
        Returns Pandas dataframe of all historical player data.
        '''

        players = pd.read_html(self.skaters_url)[0]
        players.columns = ['Rank', 'Player', 'From', 'To', 'Yrs', 'GP', 'G', \
            'A', 'PTS', '+/-', 'PIM', 'EVG', 'PPG', 'SHG', 'GWG', 'EVA', \
            'PPA', 'SHA', 'S', 'S%', 'TOI', 'ATOI']
        players.dropna(axis='rows', subset='Player', inplace=True)
        players.drop(columns={'Rank', 'ATOI'}, inplace=True)
        players = players[players['Player'] != 'Player']
        players.set_index('Player', inplace=True)
        players = players.apply(pd.to_numeric)
        
        return players

    def goalies_all_time_stats(self):
        '''
        Returns Pandas dataframe of all historical player data.
        '''

        goalies = pd.read_html(self.goalies_url)[0]
        goalies.columns = ['Rank', 'Player', 'From', 'To', 'Yrs', 'GP', 'GS', \
            'W', 'L', 'T/O', 'GA', 'SA', 'SV', 'SV%', 'GAA', 'SO', \
            'MIN', 'QS', 'QS%', 'RBS', 'GA%-', 'GSAA', 'G', 'A', 'PTS', 'PIM']
        goalies.dropna(axis='rows', subset='Player', inplace=True)
        goalies.drop(columns={'Rank'}, inplace=True)
        goalies = goalies[goalies['Player'] != 'Player']
        goalies.set_index('Player', inplace=True)
        goalies = goalies.apply(pd.to_numeric)
        
        return goalies


    def coaches_all_time_data(self):
        '''
        Returns Pandas dataframe of all historical coach data.
        '''

        coaches = pd.read_html(self.coaches_url, header=[1])[0]
        coaches.columns = ['Rank', 'Coach', 'From', 'To', 'Yrs', 'G', 'W', \
            'L', 'T', 'OL', 'PTS', 'PTS%', 'Playoff G', 'Playoff W', \
            'Playoff L', 'Playoff T', 'Playoff W-L%']
        coaches.dropna(axis='rows', subset='Coach', inplace=True)
        coaches.drop(columns={'Rank'}, inplace=True)
        coaches = coaches[coaches['Coach'] != 'Coach']
        coaches.set_index('Coach', inplace=True)
        coaches = coaches.apply(pd.to_numeric)

        return coaches


    def current_roster(self):
        '''
        Returns Pandas dataframe of current roster.
        '''

        current_roster = pd.read_html(self.current_roster_url)[3]
        current_roster['Player'] = current_roster['Player']\
            .apply(lambda x: x.split('(C)')[0].strip())
        current_roster.set_index('Player', inplace=True)
        current_roster = current_roster[['Pos', 'Age', 'Ht', 'Wt']]
        current_roster.rename(columns={
            'Pos':'Position',
            'Ht': 'Height',
            'Wt': 'Weight'
        }, inplace=True)
        current_roster['Age'] = current_roster['Age'].apply(lambda x: int(x))
        current_roster['Weight'] = current_roster['Weight'].\
            apply(lambda x: int(x))

        return current_roster

    
    def season_history(self):
        '''
        Returns Pandas dataframe of seasons.
        '''

        seasons = pd.read_html(self.team_url)[0]
        seasons = seasons[seasons['Team'] != 'Team']
        seasons.set_index('Season', inplace=True)
        seasons.drop(columns={'Lg', 'Team'}, 
                    inplace=True)

        return seasons

    def __repr__(self):
        return f"<{self.abbreviation} - {self.franchise}>"


import pytest
from sportrefpy.nba.league import NBA
from sportrefpy.nhl.league import NHL
from sportrefpy.nfl.league import NFL
from sportrefpy.mlb.league import MLB
from sportrefpy.cbb.cbb import CBB
from sportrefpy.cfb.cfb import CFB

def test_NBA():
    nba = NBA()
    assert len(nba.teams) == 30


def test_NHL():
    nhl = NHL()
    assert len(nhl.teams) == 32


def test_NFL():
    nfl = NFL()
    assert len(nfl.teams) == 32


def test_MLB():
    mlb = MLB()
    assert len(mlb.teams) == 30


def test_CBB():
    cbb = CBB()
    assert len(cbb.schools) == 485


def test_CFB():
    cfb = CFB()
    assert len(cfb.schools) == 298


import pandas as pd

from sportrefpy.nba.player import NBAPlayer
from sportrefpy.nba.team import NBAFranchise


def compare_players(players, stats, total='career'):
    '''
    Compare regular season, post season, and career totals between two players.
    '''

    players = [NBAPlayer(player) for player in players]
    if not stats:
        if total == 'career':
            comparison = pd.concat([player.career_totals() for player in players], axis=1)
            comparison.columns = [player.full_name for player in players]
        elif total == 'reg':
            comparison = pd.concat([player.regular_season_stats().sum() for player in players], axis=1)
            comparison.columns = [player.full_name for player in players]
            comparison.drop(index={'Age', 'Tm', 'Lg', 'Pos', 'FG%', '3P%', '2P%', 'eFG%', 'FT%'}, inplace=True)
        elif total == 'post':
            comparison = pd.concat([player.post_season_stats().sum() for player in players], axis=1)
            comparison.columns = [player.full_name for player in players]
            comparison.drop(index={'Age', 'Tm', 'Lg', 'Pos', 'FG%', '3P%', '2P%', 'eFG%', 'FT%'}, inplace=True)
    elif stats:
        if total == 'career':
            comparison = pd.concat([player.career_totals().loc[stats] for player in players], axis=1)
            comparison.columns = [player.full_name for player in players]
        elif total == 'reg':
            comparison = pd.concat([player.regular_season_stats().sum() for player in players], axis=1)
            comparison.columns = [player.full_name for player in players]
            comparison.drop(index={'Age', 'Tm', 'Lg', 'Pos', 'FG%', '3P%', '2P%', 'eFG%', 'FT%'}, inplace=True)
            comparison = comparison.loc[stats]
        elif total == 'post':
            comparison = pd.concat([player.post_season_stats().sum() for player in players], axis=1)
            comparison.columns = [player.full_name for player in players]
            comparison.drop(index={'Age', 'Tm', 'Lg', 'Pos', 'FG%', '3P%', '2P%', 'eFG%', 'FT%'}, inplace=True)
            comparison = comparison.loc[stats]
        # comparison['Diff'] = abs(comparison[player1] - comparison[player2])
        comparison = comparison.transpose()
        comparison.columns = [stats]
        return comparison
    else:
        raise Exception


def compare_franchises(teams):
    teams = [NBAFranchise(team) for team in teams]
    comparison = pd.concat([team.season_history()[['W', 'L']].sum() for team in teams], axis=1)
    comparison.columns = [team.franchise_name for team in teams]
    comparison = comparison.transpose()
    comparison['W%'] = comparison['W'] / (comparison['W'] + comparison['L'])
    return comparison


from sportrefpy.nba.league import NBA
from sportrefpy.nba.team import NBAFranchise

def all_players():
        players = set()
        nba = NBA()
        for team in nba.teams.keys():
            franchise = NBAFranchise(team)
            players.update(franchise.players_all_time_stats().index)
        with open('sportrefpy/assets/nba_players.txt', 'w', encoding='ascii') as file:
            for player in players:
                try:
                    file.write(f'{player}\n')
                except UnicodeEncodeError:
                    continue


import pandas as pd
from bs4 import BeautifulSoup
import requests

from sportrefpy.nba.league import NBA


class NBAFranchise(NBA):
    def __init__(self, franchise):
        super().__init__()
        self.franchise = franchise.upper()
        self.abbreviation = self.teams[self.franchise]['abbrev']
        self.franchise_name = self.teams[self.franchise]['team_name']
        self.team_url = self.teams[self.franchise]['url']
        self.players_url = self.team_url + 'players.html'
        self.coaches_url = self.team_url + 'coaches.html'
        self.seasons_url = f'{self.url}/teams/{self.abbreviation}'

    
    def players_all_time_stats(self, player=None):
        '''
        Returns Pandas dataframe of all historical player data for that team.
        '''

        players = pd.read_html(self.players_url)[0]
        players.columns = ['Rank', 'Player', 'From', 'To', 'Yrs', 'G', 'MP', \
            'FG', 'FGA', '3P', '3PA', 'FT', 'FTA', 'ORB', 'TRB', 'AST', \
            'STL', 'BLK', 'TOV', 'PF', 'PTS', 'FG%', '3P%', 'FT%', 'MPG',\
            'PPG', 'RBG', 'APG']
        players.dropna(axis='rows', subset='Player', inplace=True)
        players.drop(columns={'Rank'}, inplace=True)
        players = players[players['Player'] != 'Player']
        players.set_index('Player', inplace=True)
        players = players.apply(pd.to_numeric)

        if player is not None:
            try:
                return players.loc[player]
            except KeyError:
                return 'Player not found.'

        return players


    def coaches_all_time_data(self, coach=None):
        '''
        Returns Pandas dataframe of all historical coach data.
        '''

        coaches = pd.read_html(self.coaches_url)[0]
        coaches.columns = ['Rank', 'Coach', 'From', 'To', 'Yrs', 'G', 'W', \
            'L', 'W/L%', 'W > .500', 'Playoffs', 'Playoff G', 'Playoff W', \
            'Playoff L', 'Playoff W/L%', 'Conf', 'Champ']
        coaches.dropna(axis='rows', subset='Coach', inplace=True)
        coaches.drop(columns={'Rank'}, inplace=True)
        coaches = coaches[coaches['Coach'] != 'Coach']
        coaches.set_index('Coach', inplace=True)
        coaches = coaches.apply(pd.to_numeric)

        if coach is not None:
            try:
                return coaches.loc[coach]
            except KeyError:
                return 'Coach not found.'

        return coaches


    def roster(self, season=None):
        '''
        Returns Pandas dataframe of current roster.
        '''
        if season:
            response = requests.get(self.team_url)
            soup = BeautifulSoup(response.text, features='lxml')
            for i in soup.find_all('th', attrs={'class': 'left'}):
                if str(season) in i.find('a')['href']:
                    current_roster = pd.read_html(self.url + i.find('a')['href'])[0]
                    break
            current_roster.drop(columns={'Unnamed: 6'}, inplace=True)
            current_roster['Exp'] = current_roster['Exp'].replace('R', 0)
            current_roster['Player'] = current_roster['Player'].apply(lambda \
                x: x.split(' (TW)')[0])
            current_roster.set_index('Player', inplace=True)
            current_roster['Exp'] = current_roster['Exp'].apply(lambda x:\
                 int(x))
            current_roster['Birth Date'] = current_roster['Birth Date'].apply(\
                lambda x: pd.to_datetime(x))
            return current_roster
        else:
            return None

    
    def season_history(self, year=None):
        '''
        Returns Pandas dataframe of seasons.
        '''

        seasons = pd.read_html(self.seasons_url)[0]
        seasons = seasons[seasons['Team'] != 'Team']
        seasons.set_index('Season', inplace=True)
        seasons['Team'] = seasons['Team'].apply(lambda x: x.split('*')[0])
        seasons.drop(columns={'Unnamed: 8', 'Unnamed: 15'}, 
                    inplace=True)

        if year is not None:
            try:
                return seasons.loc[year]
            except KeyError:
                return 'Season not found.'

        return seasons
        

    def __repr__(self):
        return f"<{self.abbreviation} - {self.franchise_name}>"


import pytest
from sportrefpy.nba.team import NBAFranchise
from sportrefpy.nhl.team import NHLFranchise
from sportrefpy.nfl.team import NFLFranchise
from sportrefpy.mlb.team import MLBFranchise


def test_NBA_coach_playoffs():
    sixers = NBAFranchise('PHI')
    coaches = sixers.coaches_all_time_data()
    assert coaches.loc['Brett Brown', 'Playoffs'] == 3.0

def test_NBA_coach_winloss():
    warriors = NBAFranchise('GSW')
    coaches = warriors.coaches_all_time_data()
    assert coaches.loc['Don Nelson', 'W/L%'] == 0.488

def test_nba_coach_exists():
    heat = NBAFranchise('MIA')
    coaches = heat.coaches_all_time_data()
    with pytest.raises(KeyError): 
        coaches.loc['Pat Spoelstra']

def test_NHL_coach():
    flyers = NHLFranchise('PHI')
    coaches = flyers.coaches_all_time_data()
    assert coaches.loc['Ken Hitchcock', 'Playoff W-L%'] == 0.514


def test_NFL_coach():
    eagles = NFLFranchise('PHI')
    coaches = eagles.coaches_all_time_data()
    assert coaches.loc['Dick Vermeil', 'W-L%'] == 0.535


def test_MLB_manager():
    nym = MLBFranchise('NYM')
    managers = nym.managers_all_time_data()
    assert managers.loc['Gil Hodges', 'BestFin'] == 1


from sportrefpy.nfl.league import NFL
import pandas as pd
import numpy as np

class NFLFranchise(NFL):
    def __init__(self, franchise):
        super().__init__()
        self.abbreviation = franchise.upper()
        self.franchise = self.teams[franchise.lower()]['team_name']
        self.team_url = self.teams[franchise.lower()]['url']
        self.players_url = self.team_url + 'players.html'
        self.coaches_url = self.team_url + 'coaches.htm'
        self.current_roster_url = f'{self.url}/contracts/{self.abbreviation}.html'
    
    # def players_all_time_stats(self):
    #     '''
    #     Returns Pandas dataframe of all historical player data.
    #     '''

    #     players = pd.read_html(self.players_url)[0]
    #     players.columns = ['Rank', 'Player', 'From', 'To', 'Yrs', 'G', 'MP', \
    #         'FG', 'FGA', '3P', '3PA', 'FT', 'FTA', 'ORB', 'TRB', 'AST', \
    #         'STL', 'BLK', 'TOV', 'PF', 'PTS', 'FG%', '3P%', 'FT%', 'MPG',\
    #         'PPG', 'RBG', 'APG']
    #     players.dropna(axis='rows', subset='Player', inplace=True)
    #     players.drop(columns={'Rank'}, inplace=True)
    #     players = players[players['Player'] != 'Player']
    #     players.set_index('Player', inplace=True)
    #     players = players.apply(pd.to_numeric)

    #     return players


    def coaches_all_time_data(self):
        '''
        Returns Pandas dataframe of all historical coach data.
        '''

        coaches = pd.read_html(self.coaches_url, header=[1])[0]
        coaches['Coach'] = coaches['Coach'].apply(lambda x: x.split('+')[0])
        coaches = coaches[coaches['Coach'] != 'Coach']
        coaches.set_index('Coach', inplace=True)
        coaches = coaches.apply(pd.to_numeric)

        return coaches


    # def current_roster(self):
    #     '''
    #     Returns Pandas dataframe of current roster.
    #     '''

    #     current_roster = pd.read_html(self.current_roster_url)[0]
    #     current_roster = current_roster[current_roster.columns[:2]]
    #     current_roster.columns = ['Player', 'Age']
    #     current_roster = current_roster[current_roster['Player']\
    #          != 'Team Totals']
    #     current_roster.set_index('Player', inplace=True)
    #     current_roster['Age'] = current_roster['Age'].apply(lambda x: int(x))

    #     return current_roster

    
    def season_history(self):
        '''
        Returns Pandas dataframe of seasons.
        '''

        seasons = pd.read_html(self.team_url, header=[1])[0]
        seasons = seasons[seasons['Tm'] != 'Tm']
        seasons = seasons[seasons['T/G'] != 'Overall Rank']
        seasons['Year'] = seasons['Year'].astype(int)
        seasons.rename(columns={
            'Coaches': 'Head Coach',
            'AV': 'Top Approximate Value',
            'Passer': 'Top Passer',
            'Rusher': 'Top Rusher',
            'Receiver': 'Top Receiver'
        }, inplace=True)
        seasons.set_index('Year', inplace=True)
        seasons.drop(columns={'Tm', 'Pts', 'Yds', 'Yds.1', 'Pts.1', 'Pts±',
                            'Yds±', 'out of', 'MoV', 'SoS', 'SRS', 'OSRS', 
                            'DSRS'}, inplace=True)

        return seasons
        

    def __repr__(self):
        return f"<{self.abbreviation} - {self.franchise}>"


import pytest

from sportrefpy.nba.team import NBAFranchise
from sportrefpy.nba.analysis import compare_franchises
from sportrefpy.nhl.team import NHLFranchise
from sportrefpy.nfl.team import NFLFranchise
from sportrefpy.mlb.team import MLBFranchise
from sportrefpy.cbb.cbb import CBBSchool
from sportrefpy.cfb.cfb import CFBSchool


def test_NBA_franchise():
    sixers = NBAFranchise('PhI')
    assert sixers.franchise_name == "Philadelphia 76ers"

def test_NBA_seasons():
    celtics = NBAFranchise('boS')
    seasons = celtics.season_history()
    assert seasons.loc['2011-12', 'DRtg'] == 98.2

def test_NBA_franchise_comparison():
    comparison = compare_franchises(['PHI', 'CLE'])
    assert comparison.loc['Philadelphia 76ers', 'W'] > comparison.loc['Cleveland Cavaliers', 'W']


def test_NHL_franchise():
    golden_knights = NHLFranchise('VEG')
    assert golden_knights.franchise == "Vegas Golden Knights"

def test_NHL_seasons():
    buffalo = NHLFranchise('BUF')
    seasons = buffalo.season_history()
    assert seasons.loc['1999-00', 'Finish'] == '3rd of 5'


def test_NFL_franchise():
    kc = NFLFranchise('kan')
    assert kc.franchise == "Kansas City Chiefs"

def test_NFL_seasons():
    eagles = NFLFranchise('PHI')
    seasons = eagles.season_history()
    assert seasons.loc[2017, 'Playoffs'] == 'Won SB'


def test_MLB_franchise():
    dodgers = MLBFranchise('LAD')
    assert dodgers.franchise == "Los Angeles Dodgers"

def test_MLB_seasons():
    nyy = MLBFranchise('NYY')
    seasons = nyy.season_history()
    assert seasons.loc[1921, 'Playoffs'] == 'Lost WS (5-3)'


def test_CBB_school():
    af = CBBSchool('air-force')
    assert af.school == "Air Force Falcons"


def test_CFB_school():
    wake = CFBSchool('wake-forest')
    assert wake.school == "Wake Forest"


import requests

from bs4 import BeautifulSoup
import pandas as pd
import numpy as np
import enchant

from sportrefpy.nba.league import NBA
from sportrefpy.errors.not_found import PlayerNotFound


class NBAPlayer(NBA):
    def __init__(self, player):
        super().__init__()
        
        self.player_dict = enchant.PyPWL('sportrefpy/assets/nba_players.txt')
        first_letter = player.split()[-1][0].lower()
        players = pd.read_html(self.url + f'/players/{first_letter}')[0]
        players['Player'] = players['Player'].apply(lambda x: x.split('*')[0])
        if player in players['Player'].values:
            response = requests.get(self.url + f'/players/{first_letter}')
            soup = BeautifulSoup(response.text, features='lxml')
            for item in soup.find_all('th', attrs={'class': 'left'}):
                if player in item.text:
                    self.player_url = self.url + item.find('a')['href']
                    response = requests.get(self.player_url)
                    soup = BeautifulSoup(response.text, features='lxml')
                    self.game_log_url = self.player_url.\
                        replace('.html', '/gamelog/')
                    if soup.find_all('div', \
                        attrs={'id': 'switcher_per_game-playoffs_per_game'}):
                        self.playoffs = True
                        self.playoff_url = self.game_log_url.\
                            replace('gamelog', 'gamelog-playoffs')
                    else:
                        self.playoffs = False
                self.full_name = player
        else:
            try:
                suggestion = self.player_dict.suggest(player)[0]
                message = f'''<{player}> not found. 
Is it possible you meant {suggestion}?
Player names are case-sensitive.'''
            except:
                message = f'''<{player}> not found.
Player names are case-sensitive.'''
            raise PlayerNotFound(message)

    def regular_season_stats(self):
        '''
        Returns a players regular seasons stats by season or by career.

        It can show stats per year in total or by team, 
        if they played for multiple.
        '''
        
        if self.playoffs == True:
            stats = pd.read_html(self.player_url)[2]
            stats.drop(columns={'Unnamed: 30'}, inplace=True)
        else:
            stats = pd.read_html(self.player_url)[1]
        stats.dropna(how='all', axis='rows', inplace=True)
        stats = stats[~stats['Season'].str.contains('season|Career')]
        stats.set_index('Season', inplace=True)

        return stats

    
    def post_season_stats(self):
        '''
        Returns a players postseason seasons stats (if applicable) 
        by season or by career.
        
        It can show stats per year in total or by team, 
        if they played for multiple.
        '''
        
        if self.playoffs is False:
            return None
        else:
            stats = pd.read_html(self.player_url)[3]
            if 'Unnamed: 30' in stats.columns:
                stats.drop(columns={'Unnamed: 30'}, inplace=True)
            stats.dropna(how='all', axis='rows', inplace=True)
            stats = stats[~stats['Season'].str.contains('season|Career')]
            stats.set_index('Season', inplace=True)

            return stats

    
    def reg_season_game_log(self, season=None):
        if season:
            year = str(1 + int(season.split('-')[0]))
        games = pd.read_html(self.game_log_url + year)[-1]
        games.drop(columns=(['G', 'Unnamed: 5']), inplace=True)
        games.rename(columns={
            'Unnamed: 7': 'Result', 
            'Rk': 'G', 
            'GS': 'Start'}, 
            inplace=True)
        games = games[games['Date'] != 'Date']
        games.replace('Did Not Dress', np.nan, inplace=True)
        for column in games.columns:
                try:
                    games[column] = games[column].apply(pd.to_numeric)
                except:
                    continue
        games.set_index('G', inplace=True)

        return games


    def post_season_game_log(self):
            if self.playoffs:
                playoffs = pd.read_html(self.playoff_url)[-1]
                playoffs.drop(columns=(['G', 'Unnamed: 5']), inplace=True)
                playoffs.dropna(axis='rows', how='all', inplace=True)
                playoffs.rename(columns={
                    'Unnamed: 8': 'Result', 
                    'Rk': 'G', 
                    'GS': 'Start'}, 
                    inplace=True)
                playoffs.rename(columns={playoffs.columns[1]: 'Date'}, inplace=True)
                playoffs = playoffs[playoffs['G'] != 'Rk']
                playoffs.replace('Inactive', np.nan, inplace=True)
                playoffs.reset_index(inplace=True, drop=True)
                for column in playoffs.columns:
                        try:
                            playoffs[column] = pd.to_numeric(playoffs[column], errors='ignore')
                        except:
                            continue
                playoffs.set_index('G', inplace=True)

                return playoffs
            else:
                return None

    
    def career_totals(self):
        '''
        Find player totals (includes regular and post season)
        '''

        reg = self.regular_season_stats()
        reg.reset_index(inplace=True)
        post = self.post_season_stats()
        post.reset_index(inplace=True)
        career = reg.merge(post, how='outer')
        career = career[['G', 'GS', 'MP', 'FG', 'FGA', '3P', '3PA', '2P',\
             '2PA', 'FT', 'FTA', 'ORB', 'DRB', 'TRB', 'AST', 'STL', 'BLK', \
            'TOV', 'PF', 'PTS', 'Trp Dbl']]
        career = pd.DataFrame(career.sum())
        career.rename(columns={0: 'Total'}, inplace=True)
        return career


    def __repr__(self):
        return f"<{self.full_name}>"


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='sportrefpy',
    author='alex kahan',
    author_email='kahanscious@gmail.com',
    version='0.1.2',
    description='pull sports stats',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/alexkahan/sportrefpy',
    license='MIT',
    packages=find_packages(),
    classifiers=["Programming Language :: Python :: 3"],
    keywords='sports stats nba nfl mlb cfb cbb',
    install_requires=[
        'bs4 >= 0.0.1',
        'lxml >= 4.8.0',
        'numpy >= 1.22.3',
        'pandas >= 1.4.1',
        'pytest >= 7.1.1',
        'requests >= 2.27.1',
        'urllib3 >= 1.26.9',
        'pyenchant >=3.2.2',
    ],
    include_package_data=True,
)


import requests

from bs4 import BeautifulSoup


class NBA:   

    def __init__(self):
        self.url = 'https://www.basketball-reference.com'
        self.teams = {}

        response = requests.get(self.url + '/teams')
        soup = BeautifulSoup(response.text, features='lxml')

        for item in soup.find_all(attrs={'data-stat': 'franch_name'})[1:31]:
            self.teams[item.find('a')['href'].split('/')[-2]] = {
                "team_name": item.text,
                "abbrev": item.find('a')['href'].split('/')[-2],
                "url": self.url + item.find('a')['href'],
            }


    def franchise_codes(self):
        '''
        Print list of team codes, which are used for getting a specific franchise.
        '''
        for team in self.teams.items():
            print(f"{team[1]['abbrev']} ({team[1]['team_name']})")


import pytest
import numpy as np

from sportrefpy.nba.team import NBAFranchise
from sportrefpy.nba.player import NBAPlayer
from sportrefpy.nba.analysis import compare_players
from sportrefpy.nhl.team import NHLFranchise
from sportrefpy.mlb.team import MLBFranchise

def test_NBA_player_FGA():
    sixers = NBAFranchise('PHI')
    players = sixers.players_all_time_stats()
    assert players.loc['James Young', 'FGA'] == 14.0

def test_NBA_player_TRB():
    warriors = NBAFranchise('GSW')
    players = warriors.players_all_time_stats()
    assert players.loc['Wilt Chamberlain', 'TRB'] == 10768.0

def test_NBA_player():
    clippers = NBAFranchise('LAC')
    players = clippers.roster(2022)
    assert 'Kawhi Leonard' in players.index

def test_NBA_player_regular_season_stats():
    the_answer = NBAPlayer('Allen Iverson')
    assert the_answer.regular_season_stats().loc['1997-98', 'PTS'] == 1758.0

def test_NBA_player_post_season_stats_false():
    fo = NBAPlayer('Frank Oleynick')
    assert fo.playoffs is False and fo.post_season_stats() is None

def test_NBA_player_post_season_stats_true():
    sheed = NBAPlayer('Rasheed Wallace')
    assert sheed.post_season_stats().loc['2009-10', 'PTS'] == 147.0

def test_NBA_player_reg_season_game_log():
    kg = NBAPlayer('Kevin Garnett')
    gl = kg.reg_season_game_log('1999-00')
    assert gl.loc[2, 'FG'] == 13.0

def test_NBA_player_post_season_game_log():
    dametime = NBAPlayer('Damian Lillard')
    gl = dametime.post_season_game_log()
    assert np.isnan(gl.loc[56, 'FG'])

def test_NBA_player_comparison():
    comparison = compare_players(["Shaquille O'Neal", "Kobe Bryant"], stats='PTS')
    assert comparison.loc["Shaquille O'Neal", 'PTS'] < comparison.loc["Kobe Bryant", 'PTS']


def test_NHL_goalie():
    bruins = NHLFranchise('BOS')
    goalies = bruins.goalies_all_time_stats()
    assert goalies.loc['Gerry Cheevers', 'SV'] == 10579.0

def test_NHL_skaters():
    veg = NHLFranchise('VEG')
    skaters = veg.skaters_all_time_stats()
    assert skaters.loc['Pierre-Edouard Bellemare', 'PPG'] == 0.0

def test_NHL_player():
    panthers = NHLFranchise('FLA')
    players = panthers.current_roster()
    assert 'Aleksander Barkov' in players.index


def test_MLB_batter():
    sea = MLBFranchise('SEA')
    batters = sea.batters_all_time_stats()
    assert batters.loc['Ken Griffey Jr.', 'PA'] == 7250.0

def test_MLB_pitcher():
    bos = MLBFranchise('BOS')
    pitchers = bos.pitchers_all_time_stats()
    assert pitchers.loc['Dennis Eckersley', 'To'] == 1998
